import logging
from http.server import HTTPServer
from threading import Thread
from unittest import TestCase

from hvac import exceptions
from parameterized import parameterized
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase
from tests.utils.mock_github_request_handler import MockGithubRequestHandler


class TestGithub(HvacIntegrationTestCase, TestCase):
    TEST_GITHUB_PATH = "test-github"

    @classmethod
    def setUpClass(cls):
        try:
            super().setUpClass()

            # Configure mock server.
            cls.mock_server_port = utils.get_free_port()
            cls.mock_server = HTTPServer(
                ("localhost", cls.mock_server_port), MockGithubRequestHandler
            )

            # Start running mock server in a separate thread.
            # Daemon threads automatically shut down when the main process exits.
            cls.mock_server_thread = Thread(target=cls.mock_server.serve_forever)
            cls.mock_server_thread.daemon = True
            cls.mock_server_thread.start()
        except Exception:
            # Ensure that Vault server is taken down if setUpClass fails
            super().tearDownClass()
            raise

    def setUp(self):
        super().setUp()
        self.client.sys.enable_auth_method(
            method_type="github",
            path=self.TEST_GITHUB_PATH,
        )

    def tearDown(self):
        super().tearDown()
        self.client.sys.disable_auth_method(
            path=self.TEST_GITHUB_PATH,
        )

    @parameterized.expand(
        [
            (
                "just organization",
                True,
                "some-test-org",
                0,
                0,
                TEST_GITHUB_PATH,
            ),
        ]
    )
    def test_configure(
        self,
        test_label,
        expected_value,
        organization,
        ttl,
        max_ttl,
        mount_point,
    ):
        response = self.client.auth.github.configure(
            organization=organization,
            base_url=f"http://localhost:{self.mock_server_port}/",
            ttl=ttl,
            max_ttl=max_ttl,
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_value,
            second=bool(response),
        )

    @parameterized.expand(
        [
            ("just organization", "some-test-org", "", ""),
            ("custom ttl seconds", "some-test-org", "500s", ""),
            ("custom ttl minutes", "some-test-org", "500m", ""),
            ("custom ttl hours", "some-test-org", "500h", ""),
            ("custom max ttl", "some-test-org", "", "500s"),
        ]
    )
    def test_configure_and_read_configuration(
        self, test_label, organization, ttl, max_ttl
    ):
        config_response = self.client.auth.github.configure(
            organization=organization,
            base_url=f"http://localhost:{self.mock_server_port}/",
            ttl=ttl,
            max_ttl=max_ttl,
            mount_point=self.TEST_GITHUB_PATH,
        )
        logging.debug(f"config_response: {config_response}")
        self.assertEqual(first=204, second=config_response.status_code)

        read_config_response = self.client.auth.github.read_configuration(
            mount_point=self.TEST_GITHUB_PATH,
        )
        logging.debug(f"read_config_response: {read_config_response}")
        self.assertEqual(
            first=organization, second=read_config_response["data"]["organization"]
        )
        ttl_data_key = "token_ttl" if utils.vault_version_ge("1.2.0") else "ttl"
        max_ttl_data_key = (
            "token_max_ttl" if utils.vault_version_ge("1.2.0") else "max_ttl"
        )
        self.assertEqual(
            first=self.convert_python_ttl_value_to_expected_vault_response(
                ttl_value=ttl
            ),
            second=read_config_response["data"][ttl_data_key],
        )
        self.assertEqual(
            first=self.convert_python_ttl_value_to_expected_vault_response(
                ttl_value=max_ttl
            ),
            second=read_config_response["data"][max_ttl_data_key],
        )

    @parameterized.expand(
        [
            ("no policies", True, "hvac", None),
            ("with policies", True, "hvac", ["default"]),
        ]
    )
    def test_map_team(self, test_label, expected_value, team_name, policies):
        response = self.client.auth.github.map_team(
            team_name=team_name,
            policies=policies,
            mount_point=self.TEST_GITHUB_PATH,
        )
        self.assertEqual(
            first=expected_value,
            second=bool(response),
        )

    def test_read_team_mapping(self):
        response = self.client.auth.github.read_team_mapping(
            team_name="hvac",
            mount_point=self.TEST_GITHUB_PATH,
        )
        self.assertIn(
            member="data",
            container=response,
        )

    @parameterized.expand(
        [
            ("no policies", True, "hvac", None),
            ("with policy", True, "hvac", ["default"]),
            (
                "with policy incorrect type",
                True,
                "hvac",
                "default, root",
                exceptions.ParamValidationError,
                "unsupported policies argument provided",
            ),
            ("with policies", True, "hvac", ["default", "root"]),
        ]
    )
    def test_map_team_and_read_mapping(
        self,
        test_label,
        expected_value,
        team_name,
        policies,
        raises=False,
        exception_msg="",
    ):

        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.github.map_team(
                    team_name=team_name,
                    policies=policies,
                    mount_point=self.TEST_GITHUB_PATH,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            response = self.client.auth.github.map_team(
                team_name=team_name,
                policies=policies,
                mount_point=self.TEST_GITHUB_PATH,
            )
            self.assertEqual(
                first=expected_value,
                second=bool(response),
            )

            response = self.client.auth.github.read_team_mapping(
                team_name=team_name,
                mount_point=self.TEST_GITHUB_PATH,
            )
            if policies is None:
                expected_policies = ""
            else:
                expected_policies = ",".join(policies)

            self.assertDictEqual(
                d1=dict(key=team_name, value=expected_policies),
                d2=response["data"],
            )

    @parameterized.expand(
        [
            ("no policies", True, "hvac-user", None),
            ("with policies", True, "hvac-user", ["default"]),
        ]
    )
    def teat_map_user(self, test_label, expected_value, user_name, policies):
        response = self.client.auth.github.map_user(
            user_name=user_name,
            policies=policies,
            mount_point=self.TEST_GITHUB_PATH,
        )
        self.assertEqual(
            first=expected_value,
            second=bool(response),
        )

    def test_read_user_mapping(self):
        response = self.client.auth.github.read_user_mapping(
            user_name="hvac",
            mount_point=self.TEST_GITHUB_PATH,
        )
        self.assertIn(
            member="data",
            container=response,
        )

    @parameterized.expand(
        [
            ("no policies", True, "hvac", None),
            ("with policy", True, "hvac", ["default"]),
            (
                "with policy incorrect type",
                True,
                "hvac",
                "default, root",
                exceptions.ParamValidationError,
                "unsupported policies argument provided",
            ),
            ("with policies", True, "hvac", ["default", "root"]),
        ]
    )
    def test_map_user_and_read_mapping(
        self,
        test_label,
        expected_value,
        user_name,
        policies,
        raises=False,
        exception_msg="",
    ):

        if raises:
            with self.assertRaises(raises) as cm:
                self.client.auth.github.map_user(
                    user_name=user_name,
                    policies=policies,
                    mount_point=self.TEST_GITHUB_PATH,
                )
            self.assertIn(
                member=exception_msg,
                container=str(cm.exception),
            )
        else:
            response = self.client.auth.github.map_user(
                user_name=user_name,
                policies=policies,
                mount_point=self.TEST_GITHUB_PATH,
            )
            self.assertEqual(
                first=expected_value,
                second=bool(response),
            )

            response = self.client.auth.github.read_user_mapping(
                user_name=user_name,
                mount_point=self.TEST_GITHUB_PATH,
            )
            if policies is None:
                expected_policies = ""
            else:
                expected_policies = ",".join(policies)

            self.assertDictEqual(
                d1=dict(key=user_name, value=expected_policies),
                d2=response["data"],
            )

    @parameterized.expand(
        [
            ("valid token", "valid-token", None, None),
            (
                "invalid token not in org",
                "invalid-token",
                exceptions.InvalidRequest
                if utils.vault_version_lt("1.10.0")
                else exceptions.InternalServerError,
                "user is not part of required org",
            ),
        ]
    )
    def test_login(self, test_label, test_token, exceptions_raised, exception_msg):
        self.client.auth.github.configure(
            organization="hvac",
            base_url=f"http://localhost:{self.mock_server_port}/",
            mount_point=self.TEST_GITHUB_PATH,
        )
        if exceptions_raised is None:
            self.client.auth.github.login(
                token=test_token,
                mount_point=self.TEST_GITHUB_PATH,
            )
        else:
            with self.assertRaises(exceptions_raised) as cm:
                self.client.auth.github.login(
                    token=test_token,
                    mount_point=self.TEST_GITHUB_PATH,
                )
            self.assertIn(member=exception_msg, container=str(cm.exception))



#!/usr/bin/env python
from threading import Thread

import requests
import logging
from werkzeug.serving import make_server

from tests.utils import get_free_port

from tests.utils.mock_oauth_provider.app import create_app

logger = logging.getLogger(__name__)


class MockOauthProviderServerThread(Thread):
    def __init__(self, app_config=None, address="localhost", port="80"):
        Thread.__init__(self)
        self.address = address
        self.server_port = get_free_port()
        if app_config is None:
            app_config = {
                "SECRET_KEY": "secret",
                "SQLALCHEMY_DATABASE_URI": "sqlite:///:memory:",
                "SQLALCHEMY_TRACK_MODIFICATIONS": False,
            }
        app_config.update(
            {
                "OAUTH2_JWT_ISS": "http://{address}:{port}/oauth".format(
                    address=self.address, port=self.server_port
                ),
            }
        )
        app = create_app(app_config)
        self.srv = make_server("127.0.0.1", self.server_port, app)
        self.ctx = app.app_context()
        self.ctx.push()

    @property
    def url(self):
        return "http://{address}:{port}".format(
            address=self.address,
            port=self.server_port,
        )

    def run(self):
        logger.info("starting server")
        self.srv.serve_forever()

    def shutdown(self):
        self.srv.shutdown()


def create_user_session_and_client(
    server_url,
    oauth_redirect_uri,
    oauth_username="hvac-oauth-user",
    oauth_client_name="hvac-integration-tests",
    oauth_client_uri="https://python-hvac.org",
):
    # Persist our "user" session in between requests
    oauth_session = requests.Session()

    # Create our auth user
    create_user_response = oauth_session.post(
        url=f"{server_url}/api/user",
        data=dict(
            username=oauth_username,
        ),
    )
    create_user = create_user_response.json()
    logger.debug("create_user_response:\n%s\n" % create_user)

    # Set up the auth client details
    create_client_response = oauth_session.post(
        url=f"{server_url}/api/create_client",
        data=dict(
            client_name=oauth_client_name,
            client_uri=oauth_client_uri,
            scope="openid profile",
            redirect_uris=oauth_redirect_uri,
            grant_types="authorization_code",
            response_types="code",
            token_endpoint_auth_method="client_secret_basic",
        ),
    )
    client = create_client_response.json()
    logger.debug("create_client_data:\n%s\n" % client)
    return {
        "session": oauth_session,
        "client_id": client["client_id"],
        "client_secret": client["client_secret"],
        "discovery_url": f"{server_url}/oauth",
    }


if __name__ == "__main__":
    # Set up our test oidc provider / server's configuration and start it
    server = MockOauthProviderServerThread()
    server.start()

    # Persist our "user" session in between requests
    oauth_session = requests.Session()

    # Generic inputs
    oauth_username = "hvac-oauth-user"
    oauth_client_name = "hvac-integration-tests"
    oauth_client_uri = "https://python-hvac.org"
    oauth_redirect_uri = "https://localhost:8200/jwt-test/oidc/callback"

    # Unauthorized GET to the user API route
    unauth_get_response = oauth_session.get(f"{server.url}/api/user")
    logger.debug(
        "Unauthorized GET to the user API route:\n%s\n" % unauth_get_response.json()
    )

    # Create our user
    create_user_response = oauth_session.post(
        url=f"{server.url}/api/user",
        data=dict(
            username=oauth_username,
        ),
    )
    create_user = create_user_response.text
    logger.debug("create_user_response:\n%s\n" % create_user)

    # Create a client
    create_client_response = oauth_session.post(
        url=f"{server.url}/api/create_client",
        data=dict(
            client_name=oauth_client_name,
            client_uri=oauth_client_uri,
            scope="openid profile",
            redirect_uris=oauth_redirect_uri,
            grant_types="authorization_code",
            response_types="code",
            token_endpoint_auth_method="client_secret_basic",
        ),
    )
    client = create_client_response.json()
    logger.debug("create_client_data:\n%s\n" % client)

    # Authorized GET to the user API route after creating a client
    auth_get_response = oauth_session.get(f"{server.url}/api/user")
    logger.debug(
        "Authorized GET to the user API route after creating a client:\n%s\n"
        % auth_get_response.json()
    )

    authorize_response = oauth_session.get(
        url=f"{server.url}/oauth/authorize",
        data=dict(
            client_id=client["client_id"],
            scope="openid profile",
            response_type="code",
            nonce="abc",
        ),
    )
    # authorization = authorize_response.json()
    authorization = authorize_response.text
    logger.debug("authorize_response:\n%s\n" % authorization)

    # Finally stop our test oidc provider / server
    server.shutdown()



import logging
from unittest import TestCase

from parameterized import param, parameterized

from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestJWT(HvacIntegrationTestCase, TestCase):
    TEST_APPROLE_PATH = "jwt-test-approle"
    TEST_APPROLE_ROLE_ID = "jwt-test-role-id"
    TEST_JWT_PATH = "jwt-test"
    oidc_client_id = "hvac-client-id"
    oidc_server = None

    def setUp(self):
        super().setUp()
        self.client.sys.enable_auth_method(
            method_type="jwt",
            path=self.TEST_JWT_PATH,
        )

    def tearDown(self):
        super().tearDown()
        self.client.sys.disable_auth_method(
            path=self.TEST_JWT_PATH,
        )
        self.client.sys.disable_auth_method(
            path=self.TEST_APPROLE_PATH,
        )
        if self.oidc_server:
            self.oidc_server.shutdown()
            self.oidc_server = None

    @parameterized.expand(
        [
            param(
                "configure using vault identity OIDC",
                issuer="https://localhost:8200",
            ),
        ]
    )
    def test_configure(self, label, issuer):
        oidc_discovery_url = f"{issuer}/v1/identity/oidc"
        self.client.secrets.identity.configure_tokens_backend(
            issuer=issuer,
        )
        response = self.client.auth.jwt.configure(
            oidc_discovery_url=oidc_discovery_url,
            oidc_discovery_ca_pem="".join(
                open(utils.get_config_file_path("server-cert.pem")).readlines()
            ),
            path=self.TEST_JWT_PATH,
        )
        logging.debug("configure response: %s" % response)
        self.assertEqual(
            first=204,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            param(
                "configure using vault identity OIDC",
                issuer="https://localhost:8200",
            ),
        ]
    )
    def test_read_config(self, label, issuer):
        oidc_discovery_url = f"{issuer}/v1/identity/oidc"
        self.client.secrets.identity.configure_tokens_backend(
            issuer=issuer,
        )
        configure_response = self.client.auth.jwt.configure(
            oidc_discovery_url=oidc_discovery_url,
            oidc_discovery_ca_pem="".join(
                open(utils.get_config_file_path("server-cert.pem")).readlines()
            ),
            path=self.TEST_JWT_PATH,
        )
        logging.debug("configure response: %s" % configure_response)
        response = self.client.auth.jwt.read_config(
            path=self.TEST_JWT_PATH,
        )
        logging.debug("read_config response: %s" % response)
        self.assertEqual(
            first=oidc_discovery_url,
            second=response["data"]["oidc_discovery_url"],
        )

    @parameterized.expand(
        [
            param(
                "success",
                role_name="hvac",
                allowed_redirect_uris=["https://localhost:8200/jwt-test/callback"],
                user_claim="https://vault/user",
            ),
        ]
    )
    def test_create_role(self, label, role_name, allowed_redirect_uris, user_claim):
        response = self.client.auth.jwt.create_role(
            name=role_name,
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            bound_audiences=["1234"],
            path=self.TEST_JWT_PATH,
        )
        logging.debug("create_role response: %s" % response)
        if utils.vault_version_lt("1.11"):
            self.assertIn(
                member="data",
                container=response,
            )
        else:
            self.assertEqual(
                204,
                response.status_code,
            )

    @parameterized.expand(
        [
            param(
                "success",
                role_name="hvac",
                allowed_redirect_uris=["https://localhost:8200/jwt-test/callback"],
                user_claim="https://vault/user",
            ),
        ]
    )
    def test_read_role(self, label, role_name, allowed_redirect_uris, user_claim):
        create_role_response = self.client.auth.jwt.create_role(
            name=role_name,
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            bound_audiences=["1234"],
            path=self.TEST_JWT_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)
        response = self.client.auth.jwt.read_role(
            name=role_name,
            path=self.TEST_JWT_PATH,
        )
        logging.debug("read_role response: %s" % response)
        self.assertEqual(
            first=user_claim,
            second=response["data"]["user_claim"],
        )

    @parameterized.expand(
        [
            param(
                "success",
                role_name="hvac",
                allowed_redirect_uris=["https://localhost:8200/jwt-test/callback"],
                user_claim="https://vault/user",
            ),
        ]
    )
    def test_list_roles(self, label, role_name, allowed_redirect_uris, user_claim):
        create_role_response = self.client.auth.jwt.create_role(
            name=role_name,
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            bound_audiences=["1234"],
            path=self.TEST_JWT_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)
        response = self.client.auth.jwt.list_roles(
            path=self.TEST_JWT_PATH,
        )
        logging.debug("list_roles response: %s" % response)
        self.assertIn(
            member=role_name,
            container=response["data"]["keys"],
        )

    @parameterized.expand(
        [
            param(
                "success",
                role_name="hvac",
                allowed_redirect_uris=["https://localhost:8200/jwt-test/callback"],
                user_claim="https://vault/user",
            ),
        ]
    )
    def test_delete_role(self, label, role_name, allowed_redirect_uris, user_claim):
        create_role_response = self.client.auth.jwt.create_role(
            name=role_name,
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            bound_audiences=["1234"],
            path=self.TEST_JWT_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)
        response = self.client.auth.jwt.delete_role(
            name=role_name,
            path=self.TEST_JWT_PATH,
        )
        logging.debug("delete_role response: %s" % response)
        self.assertEqual(
            first=204,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            param(
                "success",
                issuer="https://localhost:8200",
                role_name="hvac-jwt",
                allowed_redirect_uris=["https://localhost:8200/jwt-test/oidc/callback"],
                user_claim="sub",
            ),
        ]
    )
    def test_jwt_login(
        self, label, issuer, role_name, allowed_redirect_uris, user_claim
    ):
        if "%s/" % self.TEST_APPROLE_PATH not in self.client.sys.list_auth_methods():
            self.client.sys.enable_auth_method(
                method_type="approle",
                path=self.TEST_APPROLE_PATH,
            )
        id_token_role_name = "hvac-jwt-test"
        key_name = "jwt-test-key"
        create_named_key_response = self.client.secrets.identity.create_named_key(
            name=key_name,
        )
        logging.debug("create_named_key response: %s" % create_named_key_response)
        create_or_update_role_response = (
            self.client.secrets.identity.create_or_update_role(
                name=id_token_role_name,
                key=key_name,
            )
        )
        logging.debug(
            "create_or_update_role response: %s" % create_or_update_role_response
        )
        read_role_response = self.client.secrets.identity.read_role(
            name=id_token_role_name,
        )
        logging.debug("read_role response: %s" % read_role_response)
        token_client_id = read_role_response["data"]["client_id"]
        create_named_key_response = self.client.secrets.identity.create_named_key(
            name=key_name,
            allowed_client_ids=[
                token_client_id,
            ],
        )
        logging.debug("create_named_key response: %s" % create_named_key_response)

        self.client.secrets.identity.configure_tokens_backend(
            issuer="https://localhost:8200",
        )
        response = self.client.auth.jwt.configure(
            jwks_url="https://localhost:8200/v1/identity/oidc/.well-known/keys",
            jwks_ca_pem="".join(
                open(utils.get_config_file_path("server-cert.pem")).readlines()
            ),
            path=self.TEST_JWT_PATH,
        )
        logging.debug("configure response: %s" % response)
        create_role_response = self.client.auth.jwt.create_role(
            name=role_name,
            role_type="jwt",
            allowed_redirect_uris=allowed_redirect_uris,
            user_claim=user_claim,
            bound_audiences=[token_client_id],
            path=self.TEST_JWT_PATH,
        )
        logging.debug("create_role_response: %s" % create_role_response)

        # Log in using a dummy approle role so our client token has an associated identity
        self.login_using_admin_approle_role(
            role_id=self.TEST_APPROLE_ROLE_ID,
            path=self.TEST_APPROLE_PATH,
        )
        generate_token_response = self.client.secrets.identity.generate_signed_id_token(
            name=id_token_role_name,
        )
        logging.debug("generate_token_response: %s" % generate_token_response)

        read_well_known_configurations_response = (
            self.client.secrets.identity.read_well_known_configurations()
        )
        logging.debug(
            "read_well_known_configurations_response: %s"
            % read_well_known_configurations_response
        )
        response = self.client.auth.jwt.jwt_login(
            role=role_name,
            jwt=generate_token_response["data"]["token"],
            path=self.TEST_JWT_PATH,
        )
        logging.debug("jwt_login response: %s" % response)
        self.client.token = response["auth"]["client_token"]
        self.assertIn(
            member=role_name,
            container=self.client.lookup_token()["data"]["meta"]["role"],
        )



from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac.adapters import JSONAdapter
from hvac.api.auth_methods import Mfa
from hvac.api.auth_methods.github import DEFAULT_MOUNT_POINT


class TestMfa(TestCase):
    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_configure(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/mfa_config".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        mfa = Mfa(adapter=JSONAdapter())
        response = mfa.configure(
            mount_point=mount_point,
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_configuration(self, test_label, mount_point, requests_mocker):
        expected_status_code = 200
        mock_response = {
            "lease_id": "",
            "warnings": None,
            "wrap_info": None,
            "auth": None,
            "lease_duration": 0,
            "request_id": "18ecf194-aba2-ba99-ebb5-1b90e5e231c7",
            "data": {"type": "duo"},
            "renewable": False,
        }

        mock_url = "http://localhost:8200/v1/auth/{mount_point}/mfa_config".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        mfa = Mfa(adapter=JSONAdapter())
        response = mfa.read_configuration(
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_configure_duo_access(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/duo/access".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        mfa = Mfa(adapter=JSONAdapter())
        response = mfa.configure_duo_access(
            mount_point=mount_point,
            host="someapisubdomain.python-hvac.org",
            integration_key="ikey",
            secret_key="supersecret",
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_configure_duo_behavior(self, test_label, mount_point, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{mount_point}/duo/config".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        mfa = Mfa(adapter=JSONAdapter())
        response = mfa.configure_duo_behavior(
            mount_point=mount_point, push_info="howdy"
        )
        self.assertEqual(
            first=expected_status_code,
            second=response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", DEFAULT_MOUNT_POINT),
            ("custom mount point", "cathub"),
        ]
    )
    @requests_mock.Mocker()
    def test_read_duo_behvaior_configuration(
        self, test_label, mount_point, requests_mocker
    ):
        expected_status_code = 200
        mock_response = {
            "lease_id": "",
            "warnings": None,
            "wrap_info": None,
            "auth": None,
            "lease_duration": 0,
            "request_id": "7ea734e8-bbc4-e2de-2769-d052d6a320c6",
            "data": {"username_format": "%s", "push_info": "", "user_agent": ""},
            "renewable": False,
        }

        mock_url = "http://localhost:8200/v1/auth/{mount_point}/duo/config".format(
            mount_point=mount_point,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            status_code=expected_status_code,
            json=mock_response,
        )
        mfa = Mfa(adapter=JSONAdapter())
        response = mfa.read_duo_behavior_configuration(
            mount_point=mount_point,
        )
        self.assertEqual(
            first=mock_response,
            second=response,
        )



#!/usr/bin/env python
"""Github methods module."""
from hvac import exceptions, utils
from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT = "github"


class Github(VaultApiBase):
    """GitHub Auth Method (API).

    Reference: https://www.vaultproject.io/api/auth/github/index.html
    """

    def configure(
        self,
        organization,
        base_url=None,
        ttl=None,
        max_ttl=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Configure the connection parameters for GitHub.

        This path honors the distinction between the create and update capabilities inside ACL policies.

        Supported methods:
            POST: /auth/{mount_point}/config. Produces: 204 (empty body)


        :param organization: The organization users must be part of.
        :type organization: str | unicode
        :param base_url: The API endpoint to use. Useful if you are running GitHub Enterprise or an API-compatible
            authentication server.
        :type base_url: str | unicode
        :param ttl: Duration after which authentication will be expired.
        :type ttl: str | unicode
        :param max_ttl: Maximum duration after which authentication will
            be expired.
        :type max_ttl: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the configure_method request.
        :rtype: requests.Response
        """
        params = {
            "organization": organization,
        }
        params.update(
            utils.remove_nones(
                {
                    "base_url": base_url,
                    "ttl": ttl,
                    "max_ttl": max_ttl,
                }
            )
        )
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config",
            mount_point=mount_point,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_configuration(self, mount_point=DEFAULT_MOUNT_POINT):
        """Read the GitHub configuration.

        Supported methods:
            GET: /auth/{mount_point}/config. Produces: 200 application/json


        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the read_configuration request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config",
            mount_point=mount_point,
        )
        return self._adapter.get(url=api_path)

    def map_team(self, team_name, policies=None, mount_point=DEFAULT_MOUNT_POINT):
        """Map a list of policies to a team that exists in the configured GitHub organization.

        Supported methods:
            POST: /auth/{mount_point}/map/teams/{team_name}. Produces: 204 (empty body)


        :param team_name: GitHub team name in "slugified" format
        :type team_name: str | unicode
        :param policies: Comma separated list of policies to assign
        :type policies: List[str]
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the map_github_teams request.
        :rtype: requests.Response
        """
        # First, perform parameter validation.
        if policies is None:
            policies = []
        if not isinstance(policies, list) or not all(
            isinstance(p, str) for p in policies
        ):
            error_msg = 'unsupported policies argument provided "{arg}" ({arg_type}), required type: List[str]"'
            raise exceptions.ParamValidationError(
                error_msg.format(
                    arg=policies,
                    arg_type=type(policies),
                )
            )
        # Then, perform request.
        params = {
            "value": ",".join(policies),
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/map/teams/{team_name}",
            mount_point=mount_point,
            team_name=team_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_team_mapping(self, team_name, mount_point=DEFAULT_MOUNT_POINT):
        """Read the GitHub team policy mapping.

        Supported methods:
            GET: /auth/{mount_point}/map/teams/{team_name}. Produces: 200 application/json


        :param team_name: GitHub team name
        :type team_name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the read_team_mapping request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/map/teams/{team_name}",
            mount_point=mount_point,
            team_name=team_name,
        )
        return self._adapter.get(url=api_path)

    def map_user(self, user_name, policies=None, mount_point=DEFAULT_MOUNT_POINT):
        """Map a list of policies to a specific GitHub user exists in the configured organization.

        Supported methods:
            POST: /auth/{mount_point}/map/users/{user_name}. Produces: 204 (empty body)


        :param user_name: GitHub user name
        :type user_name: str | unicode
        :param policies: Comma separated list of policies to assign
        :type policies: List[str]
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the map_github_users request.
        :rtype: requests.Response
        """
        # First, perform parameter validation.
        if policies is None:
            policies = []
        if not isinstance(policies, list) or not all(
            isinstance(p, str) for p in policies
        ):
            error_msg = 'unsupported policies argument provided "{arg}" ({arg_type}), required type: List[str]"'
            raise exceptions.ParamValidationError(
                error_msg.format(
                    arg=policies,
                    arg_type=type(policies),
                )
            )

        # Then, perform request.
        params = {
            "value": ",".join(policies),
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/map/users/{user_name}",
            mount_point=mount_point,
            user_name=user_name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_user_mapping(self, user_name, mount_point=DEFAULT_MOUNT_POINT):
        """Read the GitHub user policy mapping.

        Supported methods:
            GET: /auth/{mount_point}/map/users/{user_name}. Produces: 200 application/json


        :param user_name: GitHub user name
        :type user_name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the read_user_mapping request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/map/users/{user_name}",
            mount_point=mount_point,
            user_name=user_name,
        )
        return self._adapter.get(url=api_path)

    def login(self, token, use_token=True, mount_point=DEFAULT_MOUNT_POINT):
        """Login using GitHub access token.

        Supported methods:
            POST: /auth/{mount_point}/login. Produces: 200 application/json


        :param token: GitHub personal API token.
        :type token: str | unicode
        :param use_token: if True, uses the token in the response received from the auth request to set the "token"
            attribute on the the :py:meth:`hvac.adapters.Adapter` instance under the _adapater Client attribute.
        :type use_token: bool
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the login request.
        :rtype: dict
        """
        params = {
            "token": token,
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/login", mount_point=mount_point
        )
        return self._adapter.login(
            url=api_path,
            use_token=use_token,
            json=params,
        )



#!/usr/bin/env python
"""Constants related to the Transit secrets engine."""

import re

ALLOWED_KEY_TYPES = [
    "aes256-gcm96",
    "chacha20-poly1305",
    "ed25519",
    "ecdsa-p256",
    "ecdsa-p384",
    "ecdsa-p521",
    "rsa-2048",
    "rsa-3072",
    "rsa-4096",
]

ALLOWED_EXPORT_KEY_TYPES = [
    "encryption-key",
    "signing-key",
    "hmac-key",
]

ALLOWED_DATA_KEY_TYPES = [
    "plaintext",
    "wrapped",
]

ALLOWED_DATA_KEY_BITS = [128, 256, 512]

ALLOWED_HASH_DATA_ALGORITHMS = [
    "sha2-224",
    "sha2-256",
    "sha2-384",
    "sha2-512",
]

ALLOWED_HASH_DATA_FORMATS = ["hex", "base64"]

ALLOWED_SIGNATURE_ALGORITHMS = [
    "pss",
    "pkcs1v15",
]

ALLOWED_MARSHALING_ALGORITHMS = [
    "asn1",
    "jws",
]

# https://github.com/hashicorp/vault/pull/16549
# Either 'auto', 'hash', '-1', or any nonnegative integer.
ALLOWED_SALT_LENGTHS = re.compile(r"auto|hash|-1|\d+")



#!/usr/bin/env python
"""Gcp methods module."""
import json
import logging

from hvac import exceptions, utils
from hvac.api.vault_api_base import VaultApiBase
from hvac.constants.gcp import (
    ALLOWED_SECRETS_TYPES,
    SERVICE_ACCOUNT_KEY_ALGORITHMS,
    SERVICE_ACCOUNT_KEY_TYPES,
)

DEFAULT_MOUNT_POINT = "gcp"


class Gcp(VaultApiBase):
    """Google Cloud Secrets Engine (API).

    Reference: https://www.vaultproject.io/api/secret/gcp/index.html
    """

    def configure(
        self, credentials=None, ttl=None, max_ttl=None, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Configure shared information for the Gcp secrets engine.

        Supported methods:
            POST: /{mount_point}/config. Produces: 204 (empty body)

        :param credentials: JSON credentials (either file contents or '@path/to/file') See docs for alternative ways to
            pass in to this parameter, as well as the required permissions.
        :type credentials: str | unicode
        :param ttl: – Specifies default config TTL for long-lived credentials (i.e. service account keys). Accepts
            integer number of seconds or Go duration format string.
        :type ttl: int | str
        :param max_ttl: Specifies the maximum config TTL for long-lived credentials (i.e. service account keys). Accepts
            integer number of seconds or Go duration format string.**
        :type max_ttl: int | str
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "credentials": credentials,
                "ttl": ttl,
                "max_ttl": max_ttl,
            }
        )
        api_path = utils.format_url("/v1/{mount_point}/config", mount_point=mount_point)
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def rotate_root_credentials(self, mount_point=DEFAULT_MOUNT_POINT):
        """Rotate the GCP service account credentials used by Vault for this mount.

        A new key will be generated for the service account, replacing the internal value, and then a deletion of the
        old service account key is scheduled. Note that this does not create a new service account, only a new version
        of the service account key.

        Supported methods:
            POST: /{mount_point}/config/rotate-root. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/config/rotate-root",
            mount_point=mount_point,
        )
        return self._adapter.post(
            url=api_path,
        )

    def read_config(self, mount_point=DEFAULT_MOUNT_POINT):
        """Read the configured shared information for the Gcp secrets engine.

        Credentials will be omitted from returned data.

        Supported methods:
            GET: /{mount_point}/config. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url("/v1/{mount_point}/config", mount_point=mount_point)
        return self._adapter.get(
            url=api_path,
        )

    def create_or_update_roleset(
        self,
        name,
        project,
        bindings,
        secret_type=None,
        token_scopes=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Create a roleset or update an existing roleset.

        See roleset docs for the GCP secrets backend to learn more about what happens when you create or update a
            roleset.

        Supported methods:
            POST: /{mount_point}/roleset/{name}. Produces: 204 (empty body)

        :param name: Name of the role. Cannot be updated.
        :type name: str | unicode
        :param project: Name of the GCP project that this roleset's service account will belong to. Cannot be updated.
        :type project: str | unicode
        :param bindings: Bindings configuration string (expects HCL or JSON format in raw or base64-encoded string)
        :type bindings: str | unicode
        :param secret_type: Cannot be updated.
        :type secret_type: str | unicode
        :param token_scopes: List of OAuth scopes to assign to access_token secrets generated under this role set
            (access_token role sets only)
        :type token_scopes: list[str]
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        if secret_type is not None and secret_type not in ALLOWED_SECRETS_TYPES:
            error_msg = 'unsupported secret_type argument provided "{arg}", supported types: "{secret_type}"'
            raise exceptions.ParamValidationError(
                error_msg.format(
                    arg=secret_type,
                    secret_type=",".join(ALLOWED_SECRETS_TYPES),
                )
            )

        if isinstance(bindings, dict):
            bindings = json.dumps(bindings).replace(" ", "")
            logging.debug("bindings: %s" % bindings)

        params = {
            "project": project,
            "bindings": bindings,
        }
        params.update(
            utils.remove_nones(
                {
                    "secret_type": secret_type,
                    "token_scopes": token_scopes,
                }
            )
        )

        api_path = utils.format_url(
            "/v1/{mount_point}/roleset/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def rotate_roleset_account(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Rotate the service account this roleset uses to generate secrets.

        This also replaces the key access_token roleset. This can be used to invalidate old secrets generated by the
            roleset or fix issues if a roleset's service account (and/or keys) was changed outside of Vault (i.e.
            through GCP APIs/cloud console).

        Supported methods:
            POST: /{mount_point}/roleset/{name}/rotate. Produces: 204 (empty body)

        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/roleset/{name}/rotate",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
        )

    def rotate_roleset_account_key(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Rotate the service account key this roleset uses to generate access tokens.

        This does not recreate the roleset service account.

        Supported methods:
            POST: /{mount_point}/roleset/{name}/rotate-key. Produces: 204 (empty body)

        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/roleset/{name}/rotate-key",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
        )

    def read_roleset(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Read a roleset.

        Supported methods:
            GET: /{mount_point}/roleset/{name}. Produces: 200 application/json

        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/roleset/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_rolesets(self, mount_point=DEFAULT_MOUNT_POINT):
        """List configured rolesets.

        Supported methods:
            LIST: /{mount_point}/rolesets. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/rolesets", mount_point=mount_point
        )
        return self._adapter.list(
            url=api_path,
        )

    def delete_roleset(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing roleset by the given name.

        Supported methods:
            DELETE: /{mount_point}/roleset/{name} Produces: 200 application/json

        :param name: Name of the role.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/roleset/{name}",
            name=name,
            mount_point=mount_point,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def generate_oauth2_access_token(self, roleset, mount_point=DEFAULT_MOUNT_POINT):
        """Generate an OAuth2 token with the scopes defined on the roleset.

        This OAuth access token can be used in GCP API calls, e.g. curl -H "Authorization: Bearer $TOKEN" ...

        Supported methods:
            GET: /{mount_point}/token/{roleset}. Produces: 200 application/json

        :param roleset: Name of an roleset with secret type access_token to generate access_token under.
        :type roleset: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/token/{roleset}",
            mount_point=mount_point,
            roleset=roleset,
        )
        return self._adapter.get(
            url=api_path,
        )

    def generate_service_account_key(
        self,
        roleset,
        key_algorithm="KEY_ALG_RSA_2048",
        key_type="TYPE_GOOGLE_CREDENTIALS_FILE",
        method="POST",
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Generate Secret (IAM Service Account Creds): Service Account Key

        If using GET ('read'), the  optional parameters will be set to their defaults. Use POST if you want to specify
            different values for these params.

        :param roleset: Name of an roleset with secret type service_account_key to generate key under.
        :type roleset: str | unicode
        :param key_algorithm: Key algorithm used to generate key. Defaults to 2k RSA key You probably should not choose
            other values (i.e. 1k),
        :type key_algorithm: str | unicode
        :param key_type: Private key type to generate. Defaults to JSON credentials file.
        :type key_type: str | unicode
        :param method: Supported methods:
            POST: /{mount_point}/key/{roleset}. Produces: 200 application/json
            GET: /{mount_point}/key/{roleset}. Produces: 200 application/json
        :type method: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/key/{roleset}",
            mount_point=mount_point,
            roleset=roleset,
        )

        return self._generate_service_account_key(
            api_path, key_algorithm, key_type, method
        )

    def create_or_update_static_account(
        self,
        name,
        service_account_email,
        bindings=None,
        secret_type=None,
        token_scopes=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Create a static account or update an existing static account.

        See static account docs for the GCP secrets backend to learn more about what happens when you create or update a
            static account.

        Supported methods:
            POST: /{mount_point}/static-account/{name}. Produces: 204 (empty body)

        :param name: Name of the static account. Cannot be updated.
        :type name: str | unicode
        :param service_account_email: Email of the GCP service account to manage. Cannot be updated.
        :type service_account_email: str | unicode
        :param bindings: Bindings configuration string (expects HCL or JSON format in raw or base64-encoded string)
        :type bindings: str | unicode
        :param secret_type: Type of secret generated for this static account. Accepted values: access_token,
            service_account_key. Cannot be updated.
        :type secret_type: str | unicode
        :param token_scopes: List of OAuth scopes to assign to access_token secrets generated under this static account
            (access_token static accounts only)
        :type token_scopes: list[str]
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        if secret_type is not None and secret_type not in ALLOWED_SECRETS_TYPES:
            error_msg = 'unsupported secret_type argument provided "{arg}", supported types: "{secret_type}"'
            raise exceptions.ParamValidationError(
                error_msg.format(
                    arg=secret_type,
                    secret_type=",".join(ALLOWED_SECRETS_TYPES),
                )
            )

        if isinstance(bindings, dict):
            bindings = json.dumps(bindings).replace(" ", "")
            logging.debug("bindings: %s" % bindings)

        params = {
            "service_account_email": service_account_email,
        }
        params.update(
            utils.remove_nones(
                {
                    "bindings": bindings,
                    "secret_type": secret_type,
                    "token_scopes": token_scopes,
                }
            )
        )
        api_path = utils.format_url(
            "/v1/{mount_point}/static-account/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def rotate_static_account_key(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Rotate the service account key this static account uses to generate access tokens.

        This does not recreate the service account.

        Supported methods:
            POST: /{mount_point}/static-account/{name}/rotate-key. Produces: 204 (empty body)

        :param name: Name of the static account.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/static-account/{name}/rotate-key",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
        )

    def read_static_account(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Read a static account.

        Supported methods:
            GET: /{mount_point}/static-account/{name}. Produces: 200 application/json

        :param name: Name of the static account.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/static-account/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_static_accounts(self, mount_point=DEFAULT_MOUNT_POINT):
        """List configured static accounts.

        Supported methods:
            LIST: /{mount_point}/static-accounts. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/static-accounts", mount_point=mount_point
        )
        return self._adapter.list(
            url=api_path,
        )

    def delete_static_account(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing static account by the given name.

        Supported methods:
            DELETE: /{mount_point}/static-account/{name} Produces: 204 (empty body)

        :param name: Name of the static account.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/static-account/{name}",
            name=name,
            mount_point=mount_point,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def generate_static_account_oauth2_access_token(
        self, name, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Generate an OAuth2 token with the scopes defined on the static account.

        This OAuth access token can be used in GCP API calls, e.g. curl -H "Authorization: Bearer $TOKEN" ...

        Supported methods:
            GET: /{mount_point}/static-account/{name}/token. Produces: 200 application/json

        :param name: Name of a static account with secret type access_token to generate access_token under.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/static-account/{name}/token",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def generate_static_account_service_account_key(
        self,
        name,
        key_algorithm="KEY_ALG_RSA_2048",
        key_type="TYPE_GOOGLE_CREDENTIALS_FILE",
        method="POST",
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Generate Secret (IAM Service Account Creds): Service Account Key

        If using GET ('read'), the  optional parameters will be set to their defaults. Use POST if you want to specify
            different values for these params.

        :param name: Name of a static account with secret type service_account_key to generate key under.
        :type name: str | unicode
        :param key_algorithm: Key algorithm used to generate key. Defaults to 2k RSA key You probably should not choose
            other values (i.e. 1k),
        :type key_algorithm: str | unicode
        :param key_type: Private key type to generate. Defaults to JSON credentials file.
        :type key_type: str | unicode
        :param method: Supported methods:
            POST: /v1/{mount_point}/static-account/{name}/key. Produces: 200 application/json
            GET: /v1/{mount_point}/static-account/{name}/key. Produces: 200 application/json
        :type method: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/static-account/{name}/key",
            mount_point=mount_point,
            name=name,
        )

        return self._generate_service_account_key(
            api_path, key_algorithm, key_type, method
        )

    def create_or_update_impersonated_account(
        self,
        name,
        service_account_email,
        token_scopes=None,
        ttl=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Create an impersonated account or update an existing impersonated account.

        See impersonated account docs for the GCP secrets backend to learn more about what happens when you create or update an
            impersonated account.

        Supported methods:
            POST: /{mount_point}/impersonated-account/{name}. Produces: 204 (empty body)

        :param name: Name of the impersonated account. Cannot be updated.
        :type name: str | unicode
        :param service_account_email: Email of the GCP service account to manage. Cannot be updated.
        :type service_account_email: str | unicode
        :param token_scopes: List of OAuth scopes to assign to access tokens generated under this impersonated account
        :type token_scopes: list[str]
        :param ttl: Lifetime of the token generated. Defaults to 1 hour and is limited to a maximum of 12 hours.
            Uses duration format strings.
        :type ttl: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            "service_account_email": service_account_email,
        }
        params.update(
            utils.remove_nones(
                {
                    "token_scopes": token_scopes,
                    "ttl": ttl,
                }
            )
        )
        api_path = utils.format_url(
            "/v1/{mount_point}/impersonated-account/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_impersonated_account(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Read an impersonated account.

        Supported methods:
            GET: /{mount_point}/impersonated-account/{name}. Produces: 200 application/json

        :param name: Name of the impersonated account.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/impersonated-account/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_impersonated_accounts(self, mount_point=DEFAULT_MOUNT_POINT):
        """List configured impersonated accounts.

        Supported methods:
            LIST: /{mount_point}/impersonated-accounts. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/impersonated-accounts", mount_point=mount_point
        )
        return self._adapter.list(
            url=api_path,
        )

    def delete_impersonated_account(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing impersonated account by the given name.

        Supported methods:
            DELETE: /{mount_point}/impersonated-account/{name} Produces: 204 (empty body)

        :param name: Name of the impersonated account.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/impersonated-account/{name}",
            name=name,
            mount_point=mount_point,
        )
        return self._adapter.delete(
            url=api_path,
        )

    def generate_impersonated_account_oauth2_access_token(
        self, name, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Generate an OAuth2 token with the scopes defined on the impersonated account.

        This OAuth access token can be used in GCP API calls, e.g. curl -H "Authorization: Bearer $TOKEN" ...

        Supported methods:
            GET: /{mount_point}/impersonated-account/{name}/token. Produces: 200 application/json

        :param name: Name of the impersonated account to generate an access token under.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/{mount_point}/impersonated-account/{name}/token",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def _generate_service_account_key(
        self,
        api_path,
        key_algorithm="KEY_ALG_RSA_2048",
        key_type="TYPE_GOOGLE_CREDENTIALS_FILE",
        method="POST",
    ):
        if method == "POST":
            if key_algorithm not in SERVICE_ACCOUNT_KEY_ALGORITHMS:
                error_msg = 'unsupported key_algorithm argument provided "{arg}", supported algorithms: "{algorithms}"'
                raise exceptions.ParamValidationError(
                    error_msg.format(
                        arg=key_algorithm,
                        algorithms=",".join(SERVICE_ACCOUNT_KEY_ALGORITHMS),
                    )
                )
            if key_type not in SERVICE_ACCOUNT_KEY_TYPES:
                error_msg = 'unsupported key_type argument provided "{arg}", supported types: "{key_types}"'
                raise exceptions.ParamValidationError(
                    error_msg.format(
                        arg=key_type,
                        key_types=",".join(SERVICE_ACCOUNT_KEY_TYPES),
                    )
                )

            params = {
                "key_algorithm": key_algorithm,
                "key_type": key_type,
            }

            response = self._adapter.post(
                url=api_path,
                json=params,
            )
        elif method == "GET":
            response = self._adapter.get(
                url=api_path,
            )
        else:
            error_message = '"method" parameter provided invalid value; POST or GET allowed, "{method}" provided'.format(
                method=method
            )
            raise exceptions.ParamValidationError(error_message)

        return response



import logging
from unittest import TestCase, skipIf

from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


@skipIf(not utils.is_enterprise(), "Namespaces only supported with Enterprise Vault")
class TestNamespace(HvacIntegrationTestCase, TestCase):
    def test_list_namespaces(self):
        test_namespace_name = "python-hvac"
        create_namespace_response = self.client.sys.create_namespace(
            path=test_namespace_name
        )
        logging.debug("create_namespace_response: %s" % create_namespace_response)

        # Verify the namespace we just created is retrievable in a listing.
        list_namespaces_response = self.client.sys.list_namespaces()
        logging.debug("list_namespaces_response: %s" % list_namespaces_response)
        self.assertIn(
            member="%s/" % test_namespace_name,
            container=list_namespaces_response["data"]["keys"],
        )



from unittest import TestCase

import requests_mock
from parameterized import parameterized

from hvac import Client


class TestAwsEc2Methods(TestCase):
    """Unit tests providing coverage for AWS (EC2) auth backend-related methods/routes."""

    @parameterized.expand(
        [
            ("default mount point", None),
            ("custom mount point", "aws"),
        ]
    )
    @requests_mock.Mocker()
    def test_auth_ec2(self, test_label, mount_point, requests_mocker):
        mock_response = {
            "auth": {
                "accessor": "accessor-1234-5678-9012-345678901234",
                "client_token": "cltoken-1234-5678-9012-345678901234",
                "lease_duration": 10000,
                "metadata": {
                    "account_id": "12345678912",
                    "ami_id": "ami-someami",
                    "instance_id": "i-instanceid",
                    "nonce": "thenonce-1234-5678-9012-345678901234",
                    "region": "us-east-1",
                    "role": "custom_role",
                    "role_tag_max_ttl": "0s",
                },
                "policies": ["default", "custom_role"],
                "renewable": True,
            },
            "data": None,
            "lease_duration": 0,
            "lease_id": "",
            "renewable": False,
            "request_id": "requesti-1234-5678-9012-345678901234",
            "warnings": [],
            "wrap_info": None,
        }
        mock_url = "http://localhost:8200/v1/auth/{}/login".format(
            "aws" if mount_point is None else mount_point
        )
        requests_mocker.register_uri(method="POST", url=mock_url, json=mock_response)
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.ec2_login(pkcs7="mock_pcks7")
        else:
            actual_response = client.auth.aws.ec2_login(
                pkcs7="mock_pcks7", mount_point=mount_point
            )

        # ensure we received our mock response data back successfully
        self.assertEqual(mock_response, actual_response)

    @parameterized.expand(
        [
            ("default mount point", None),
            ("custom mount point", "aws"),
        ]
    )
    @requests_mock.Mocker()
    def test_create_vault_ec2_client_configuration(
        self, test_label, mount_point, requests_mocker
    ):
        test_access_key = "AKIAABCDEFGUE1234567"
        test_secret_key = "thisisasecretyall"
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/config/client".format(
            "aws" if mount_point is None else mount_point
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.configure(
                access_key=test_access_key,
                secret_key=test_secret_key,
            )
        else:
            actual_response = client.auth.aws.configure(
                access_key=test_access_key,
                secret_key=test_secret_key,
                mount_point=mount_point,
            )

        self.assertEqual(first=expected_status_code, second=actual_response.status_code)

    @parameterized.expand(
        [
            ("default mount point", None),
            ("custom mount point", "aws"),
        ]
    )
    @requests_mock.Mocker()
    def test_get_vault_ec2_client_configuration(
        self, test_label, mount_point, requests_mocker
    ):
        mock_response = {
            "data": {
                "secret_key": "vCtSM8ZUEQ3mOFVlYPBQkf2sO6F/W7a5TVzrl3Oj",
                "access_key": "VKIAJBRHKH6EVTTNXDHA",
                "endpoint": "",
                "iam_endpoint": "",
                "sts_endpoint": "",
                "iam_server_id_header_value": "",
            }
        }
        expected_status_code = 200
        mock_url = "http://localhost:8200/v1/auth/{}/config/client".format(
            "aws" if mount_point is None else mount_point
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            json=mock_response,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.read_config()
        else:
            actual_response = client.auth.aws.read_config(mount_point=mount_point)

        self.assertEqual(
            first=mock_response.get("data"),
            second=actual_response,
        )

    @parameterized.expand(
        [
            ("default mount point", None),
            ("custom mount point", "aws"),
        ]
    )
    @requests_mock.Mocker()
    def test_delete_vault_ec2_client_configuration(
        self, test_label, mount_point, requests_mocker
    ):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/config/client".format(
            "aws" if mount_point is None else mount_point
        )
        requests_mocker.register_uri(
            method="DELETE",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.delete_config()
        else:
            actual_response = client.auth.aws.delete_config(mount_point=mount_point)

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "my-cool-cert-1"),
            ("custom mount point", "aws", "my-cool-cert-2"),
        ]
    )
    @requests_mock.Mocker()
    def test_create_vault_ec2_certificate_configuration(
        self, test_label, mount_point, cert_name, requests_mocker
    ):
        test_cert_info = "this is some test cert info"
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/config/certificate/{}".format(
            "aws" if mount_point is None else mount_point,
            cert_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.create_certificate_configuration(
                cert_name=cert_name,
                aws_public_cert=test_cert_info,
            )
        else:
            actual_response = client.auth.aws.create_certificate_configuration(
                cert_name=cert_name,
                aws_public_cert=test_cert_info,
                mount_point=mount_point,
            )

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "my-cool-cert-1"),
            ("custom mount point", "aws", "my-cool-cert-2"),
        ]
    )
    @requests_mock.Mocker()
    def test_get_vault_ec2_certificate_configuration(
        self, test_label, mount_point, cert_name, requests_mocker
    ):
        mock_response = {
            "data": {
                "aws_public_cert": "-----BEGIN CERTIFICATE-----\nblah blah9K\n-----END CERTIFICATE-----\n",
                "type": "pkcs7",
            }
        }
        expected_status_code = 200
        mock_url = "http://localhost:8200/v1/auth/{}/config/certificate/{}".format(
            "aws" if mount_point is None else mount_point,
            cert_name,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            json=mock_response,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.read_certificate_configuration(
                cert_name=cert_name,
            )
        else:
            actual_response = client.auth.aws.read_certificate_configuration(
                cert_name=cert_name, mount_point=mount_point
            )

        self.assertEqual(
            first=mock_response.get("data"),
            second=actual_response,
        )

    @parameterized.expand(
        [
            ("default mount point", None),
            ("custom mount point", "aws"),
        ]
    )
    @requests_mock.Mocker()
    def test_list_vault_ec2_certificate_configurations(
        self, test_label, mount_point, requests_mocker
    ):
        mock_response = {"data": {"keys": ["cert1"]}}

        expected_status_code = 200
        mock_url = "http://localhost:8200/v1/auth/{}/config/certificates".format(
            "aws" if mount_point is None else mount_point,
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            json=mock_response,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.list_certificate_configurations()
        else:
            actual_response = client.auth.aws.list_certificate_configurations(
                mount_point=mount_point
            )

        self.assertEqual(
            first=mock_response.get("data"),
            second=actual_response,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "my-role-1"),
            ("custom mount point", "aws", "my-role-2"),
        ]
    )
    @requests_mock.Mocker()
    def test_create_ec2_role(self, test_label, mount_point, role_name, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}".format(
            "aws" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.create_role(role=role_name)
        else:
            actual_response = client.auth.aws.create_role(
                role=role_name, mount_point=mount_point
            )

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "my-role-1"),
            ("custom mount point", "aws", "my-role-2"),
        ]
    )
    @requests_mock.Mocker()
    def test_get_ec2_role(self, test_label, mount_point, role_name, requests_mocker):
        mock_response = {
            "data": {
                "bound_ami_id": ["ami-fce36987"],
                "role_tag": "",
                "policies": ["default", "dev", "prod"],
                "max_ttl": 1800000,
                "disallow_reauthentication": False,
                "allow_instance_migration": False,
            }
        }
        expected_status_code = 200
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}".format(
            "aws" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="GET",
            url=mock_url,
            json=mock_response,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.read_role(role=role_name)
        else:
            actual_response = client.auth.aws.read_role(
                role=role_name, mount_point=mount_point
            )

        self.assertEqual(
            first=mock_response.get("data"),
            second=actual_response,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "my-role-1"),
            ("custom mount point", "aws", "my-role-2"),
        ]
    )
    @requests_mock.Mocker()
    def test_delete_ec2_role(self, test_label, mount_point, role_name, requests_mocker):
        expected_status_code = 204
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}".format(
            "aws" if mount_point is None else mount_point,
            role_name,
        )
        requests_mocker.register_uri(
            method="DELETE",
            url=mock_url,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.delete_role(role=role_name)
        else:
            actual_response = client.auth.aws.delete_role(
                role=role_name, mount_point=mount_point
            )

        self.assertEqual(
            first=expected_status_code,
            second=actual_response.status_code,
        )

    @parameterized.expand(
        [
            ("default mount point", None),
            ("custom mount point", "aws"),
        ]
    )
    @requests_mock.Mocker()
    def test_list_ec2_roles(self, test_label, mount_point, requests_mocker):
        mock_response = {"data": {"keys": ["dev-role", "prod-role"]}}
        expected_status_code = 200
        mock_url = "http://localhost:8200/v1/auth/{}/roles".format(
            "aws" if mount_point is None else mount_point,
        )
        requests_mocker.register_uri(
            method="LIST",
            url=mock_url,
            json=mock_response,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.list_roles()
        else:
            actual_response = client.auth.aws.list_roles(mount_point=mount_point)

        self.assertEqual(
            first=mock_response.get("data"),
            second=actual_response,
        )

    @parameterized.expand(
        [
            ("default mount point", None, "my-role-1"),
            ("custom mount point", "aws", "my-role-2"),
        ]
    )
    @requests_mock.Mocker()
    def test_create_ec2_role_tag(
        self, test_label, mount_point, role_name, requests_mocker
    ):
        mock_response = {
            "data": {
                "tag_value": "v1:09Vp0qGuyB8=:r=dev-role:p=default,dev-api:d=false:t=300h0m0s:uPLKCQxqsefRhrp1qmVa1wsQVUXXJG8UZP/pJIdVyOI=",
                "tag_key": "VaultRole",
            }
        }
        expected_status_code = 200
        mock_url = "http://localhost:8200/v1/auth/{}/role/{}/tag".format(
            "aws" if mount_point is None else mount_point, role_name
        )
        requests_mocker.register_uri(
            method="POST",
            url=mock_url,
            json=mock_response,
            status_code=expected_status_code,
        )
        client = Client()

        if mount_point is None:
            actual_response = client.auth.aws.create_role_tags(
                role=role_name,
            )
        else:
            actual_response = client.auth.aws.create_role_tags(
                role=role_name, mount_point=mount_point
            )

        self.assertEqual(
            first=mock_response,
            second=actual_response,
        )



import logging
from unittest import TestCase

from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


class TestInit(HvacIntegrationTestCase, TestCase):
    def test_read_init_status(self):
        read_response = self.client.sys.read_init_status()
        logging.debug("read_response: %s" % read_response)
        self.assertTrue(expr=read_response["initialized"])



import pytest

from unittest import TestCase

from unittest import mock
from unittest.mock import MagicMock, Mock
from parameterized import parameterized

from hvac.api.secrets_engines.kv import Kv
from hvac.api.secrets_engines.kv_v1 import KvV1
from hvac.api.secrets_engines.kv_v2 import KvV2

from hvac import exceptions


class TestKv(TestCase):
    def test_v1_property(self):
        mock_adapter = MagicMock()
        kv = Kv(adapter=mock_adapter)
        self.assertIsInstance(
            obj=kv.v1,
            cls=KvV1,
        )

    def test_v2_property(self):
        mock_adapter = MagicMock()
        kv = Kv(adapter=mock_adapter)
        self.assertIsInstance(
            obj=kv.v2,
            cls=KvV2,
        )

    @parameterized.expand(
        [
            ("v1", "1"),
            ("v2", "2"),
            ("v3", "3", ValueError),
            ("invalid version", "12345", ValueError),
        ]
    )
    def test_default_kv_version_setter(self, test_label, version, raises=False):
        version_class_map = {
            "1": KvV1,
            "2": KvV2,
        }
        mock_adapter = MagicMock()
        kv = Kv(adapter=mock_adapter)

        if raises:
            with self.assertRaises(raises):
                kv.default_kv_version = version
        else:
            kv.default_kv_version = version
            self.assertIsInstance(
                obj=getattr(kv, "v%s" % version),
                cls=version_class_map.get(version),
            )

    def test_getattr(self):
        mock_adapter = MagicMock()
        kv = Kv(adapter=mock_adapter, default_kv_version="1")
        self.assertEqual(
            first=kv.read_secret,
            second=kv.v1.read_secret,
        )
        kv = Kv(adapter=mock_adapter, default_kv_version="2")
        self.assertEqual(
            first=kv.read_secret_version,
            second=kv.v2.read_secret_version,
        )

        kv._default_kv_version = 0
        with self.assertRaises(AttributeError):
            assert kv.read_secret


class TestKv2:
    # TODO: v3.0.0 - remove this (there should be no more warning, the default will be set statically)
    @pytest.mark.parametrize("raise_on_del", [None, True, False])
    def test_kv2_raise_on_deleted_warning(self, raise_on_del):
        mock_adapter = MagicMock()
        kv = Mock(wraps=Kv(adapter=mock_adapter, default_kv_version="2"))

        for method in [
            kv.read_secret,
            kv.read_secret_version,
            kv.v2.read_secret,
            kv.v2.read_secret_version,
        ]:
            with mock.patch("warnings.warn") as w:
                p = "secret_path"
                method(p, raise_on_deleted_version=raise_on_del)

                if raise_on_del is None:
                    assert w.call_count == 1
                    assert "category" in w.call_args[1]
                    assert w.call_args[1]["category"] == DeprecationWarning
                    # TODO: in py3.8+: assert "category" in w.call_args.kwargs
                    # TODO: in py3.8+: assert w.call_args.kwargs["category"] == DeprecationWarning
                else:
                    assert w.assert_not_called

    @pytest.mark.parametrize(
        ("json", "recoverable"),
        [
            (None, False),
            ({}, False),
            ({"data": {"metadata": {"deletion_time": ""}}}, False),
            ({"data": {"metadata": {"deletion_time": "anything"}}}, True),
        ],
    )
    @pytest.mark.parametrize("raise_on_del", [True, False])
    def test_kv2_raise_on_deleted(self, raise_on_del, json, recoverable):
        def _getem(*args, **kwargs):
            raise exceptions.InvalidPath(json=json)

        mock_adapter = MagicMock(get=_getem)
        kv = Mock(wraps=Kv(adapter=mock_adapter, default_kv_version="2"))

        for method in [
            kv.read_secret,
            kv.read_secret_version,
            kv.v2.read_secret,
            kv.v2.read_secret_version,
        ]:
            p = "secret_path"
            should_raise = raise_on_del or not recoverable

            if should_raise:
                with pytest.raises(exceptions.InvalidPath):
                    method(p, raise_on_deleted_version=raise_on_del)
            else:
                r = method(p, raise_on_deleted_version=raise_on_del)
                assert r is json



#!/usr/bin/env python
"""Okta methods module."""
from hvac import utils
from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT = "okta"


class Okta(VaultApiBase):
    """Okta Auth Method (API).

    Reference: https://www.vaultproject.io/api/auth/okta/index.html
    """

    def configure(
        self,
        org_name,
        api_token=None,
        base_url=None,
        ttl=None,
        max_ttl=None,
        bypass_okta_mfa=None,
        mount_point=DEFAULT_MOUNT_POINT,
    ):
        """Configure the connection parameters for Okta.

        This path honors the distinction between the create and update capabilities inside ACL policies.

        Supported methods:
            POST: /auth/{mount_point}/config. Produces: 204 (empty body)


        :param org_name: Name of the organization to be used in the Okta API.
        :type org_name: str | unicode
        :param api_token: Okta API token. This is required to query Okta for user group membership. If this is not
            supplied only locally configured groups will be enabled.
        :type api_token: str | unicode
        :param base_url:  If set, will be used as the base domain for API requests.  Examples are okta.com,
            oktapreview.com, and okta-emea.com.
        :type base_url: str | unicode
        :param ttl: Duration after which authentication will be expired.
        :type ttl: str | unicode
        :param max_ttl: Maximum duration after which authentication will be expired.
        :type max_ttl: str | unicode
        :param bypass_okta_mfa: Whether to bypass an Okta MFA request. Useful if using one of Vault's built-in MFA
            mechanisms, but this will also cause certain other statuses to be ignored, such as PASSWORD_EXPIRED.
        :type bypass_okta_mfa: bool
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            "org_name": org_name,
        }
        params.update(
            utils.remove_nones(
                {
                    "api_token": api_token,
                    "base_url": base_url,
                    "ttl": ttl,
                    "max_ttl": max_ttl,
                    "bypass_okta_mfa": bypass_okta_mfa,
                }
            )
        )
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config", mount_point=mount_point
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_config(self, mount_point=DEFAULT_MOUNT_POINT):
        """Read the Okta configuration.

        Supported methods:
            GET: /auth/{mount_point}/config. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/config", mount_point=mount_point
        )
        return self._adapter.get(
            url=api_path,
        )

    def list_users(self, mount_point=DEFAULT_MOUNT_POINT):
        """List the users configured in the Okta method.

        Supported methods:
            LIST: /auth/{mount_point}/users. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/users", mount_point=mount_point
        )
        return self._adapter.list(
            url=api_path,
        )

    def register_user(
        self, username, groups=None, policies=None, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Register a new user and maps a set of policies to it.

        Supported methods:
            POST: /auth/{mount_point}/users/{username}. Produces: 204 (empty body)

        :param username: Name of the user.
        :type username: str | unicode
        :param groups: List or comma-separated string of groups associated with the user.
        :type groups: list
        :param policies: List or comma-separated string of policies associated with the user.
        :type policies: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            "username": username,
        }
        params.update(
            utils.remove_nones(
                {
                    "groups": groups,
                    "policies": policies,
                }
            )
        )
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/users/{username}",
            mount_point=mount_point,
            username=username,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_user(self, username, mount_point=DEFAULT_MOUNT_POINT):
        """Read the properties of an existing username.

        Supported methods:
            GET: /auth/{mount_point}/users/{username}. Produces: 200 application/json

        :param username: Username for this user.
        :type username: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        params = {
            "username": username,
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/users/{username}",
            mount_point=mount_point,
            username=username,
        )
        return self._adapter.get(
            url=api_path,
            json=params,
        )

    def delete_user(self, username, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing username from the method.

        Supported methods:
            DELETE: /auth/{mount_point}/users/{username}. Produces: 204 (empty body)

        :param username: Username for this user.
        :type username: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            "username": username,
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/users/{username}",
            mount_point=mount_point,
            username=username,
        )
        return self._adapter.delete(
            url=api_path,
            json=params,
        )

    def list_groups(self, mount_point=DEFAULT_MOUNT_POINT):
        """List the groups configured in the Okta method.

        Supported methods:
            LIST: /auth/{mount_point}/groups. Produces: 200 application/json

        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/groups", mount_point=mount_point
        )
        return self._adapter.list(
            url=api_path,
        )

    def register_group(self, name, policies=None, mount_point=DEFAULT_MOUNT_POINT):
        """Register a new group and maps a set of policies to it.

        Supported methods:
            POST: /auth/{mount_point}/groups/{name}. Produces: 204 (empty body)

        :param name: The name of the group.
        :type name: str | unicode
        :param policies: The list or comma-separated string of policies associated with the group.
        :type policies: list
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = utils.remove_nones(
            {
                "policies": policies,
            }
        )
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/groups/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.post(
            url=api_path,
            json=params,
        )

    def read_group(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Read the properties of an existing group.

        Supported methods:
            GET: /auth/{mount_point}/groups/{name}. Produces: 200 application/json

        :param name: The name for the group.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The JSON response of the request.
        :rtype: dict
        """
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/groups/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.get(
            url=api_path,
        )

    def delete_group(self, name, mount_point=DEFAULT_MOUNT_POINT):
        """Delete an existing group from the method.

        Supported methods:
            DELETE: /auth/{mount_point}/groups/{name}. Produces: 204 (empty body)

        :param name: The name for the group.
        :type name: str | unicode
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the request.
        :rtype: requests.Response
        """
        params = {
            "name": name,
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/groups/{name}",
            mount_point=mount_point,
            name=name,
        )
        return self._adapter.delete(
            url=api_path,
            json=params,
        )

    def login(
        self, username, password, use_token=True, mount_point=DEFAULT_MOUNT_POINT
    ):
        """Login with the username and password.

        Supported methods:
            POST: /auth/{mount_point}/login/{username}. Produces: 200 application/json

        :param username: Username for this user.
        :type username: str | unicode
        :param password: Password for the authenticating user.
        :type password: str | unicode
        :param use_token: if True, uses the token in the response received from the auth request to set the "token"
            attribute on the the :py:meth:`hvac.adapters.Adapter` instance under the _adapater Client attribute.
        :type use_token: bool
        :param mount_point: The "path" the method/backend was mounted on.
        :type mount_point: str | unicode
        :return: The response of the login request.
        :rtype: dict
        """
        params = {
            "username": username,
            "password": password,
        }
        api_path = utils.format_url(
            "/v1/auth/{mount_point}/login/{username}",
            mount_point=mount_point,
            username=username,
        )
        return self._adapter.login(
            url=api_path,
            use_token=use_token,
            json=params,
        )



#!/usr/bin/env python
"""Constants related to the APPROLE auth method."""

DEFAULT_MOUNT_POINT = "approle"
ALLOWED_TOKEN_TYPES = [
    "service",
    "batch",
    "default",
    "default-service",
    "default-batch",
]



from unittest import TestCase
from unittest import skipIf

from parameterized import parameterized

from hvac import exceptions
from tests import utils
from tests.utils.hvac_integration_test_case import HvacIntegrationTestCase


@skipIf(
    utils.vault_version_lt("0.6.2"), "AppRole endpoints standardized in version 0.6.2"
)
class TestAppRole(HvacIntegrationTestCase, TestCase):
    TEST_MOUNT_POINT = "approle-test"
    TEST_ROLE_NAME = "testrole"
    TEST_ROLE_ID = "test_role_id"
    TEST_SECRET_ID = "custom_secret"

    def setUp(self):
        super().setUp()
        if "%s/" % self.TEST_MOUNT_POINT not in self.client.sys.list_auth_methods():
            self.client.sys.enable_auth_method(
                method_type="approle",
                path=self.TEST_MOUNT_POINT,
            )
        _ = self.client.auth.approle.create_or_update_approle(
            role_name=self.TEST_ROLE_NAME,
            token_policies=["default"],
            mount_point=self.TEST_MOUNT_POINT,
        )
        _ = self.client.auth.approle.update_role_id(
            role_name=self.TEST_ROLE_NAME,
            role_id=self.TEST_ROLE_ID,
            mount_point=self.TEST_MOUNT_POINT,
        )

    def tearDown(self):
        super().tearDown()
        self.client.sys.disable_auth_method(path=self.TEST_MOUNT_POINT)

    def _secret_id(self):
        secret_id_response = self.client.auth.approle.generate_secret_id(
            role_name=self.TEST_ROLE_NAME,
            cidr_list=["127.0.0.1/32"],
            mount_point=self.TEST_MOUNT_POINT,
        )
        return secret_id_response["data"]

    @parameterized.expand(
        [
            ("create test role", "default", None),
            ("bad token type", "bad_token", exceptions.ParamValidationError),
        ]
    )
    def test_create_or_update_approle(self, test_label, token_type, raises):
        if raises is not None:
            with self.assertRaises(raises) as cm:
                self.client.auth.approle.create_or_update_approle(
                    role_name="testrole2",
                    token_policies=["default"],
                    token_type=token_type,
                    mount_point=self.TEST_MOUNT_POINT,
                )
            self.assertIn(member="unsupported token_type", container=str(cm.exception))
        else:
            response = self.client.auth.approle.create_or_update_approle(
                role_name="testrole2",
                token_policies=["default"],
                mount_point=self.TEST_MOUNT_POINT,
            )

            self.assertEqual(first=bool(response), second=True)

    def test_list_roles(self):
        response = self.client.auth.approle.list_roles(
            mount_point=self.TEST_MOUNT_POINT
        )

        self.assertEqual(first=len(response["data"]["keys"]), second=1)

    def test_read_role(self):
        response = self.client.auth.approle.read_role(
            role_name=self.TEST_ROLE_NAME, mount_point=self.TEST_MOUNT_POINT
        )

        self.assertEqual(first=response["data"]["token_type"], second="default")

    def test_delete_role(self):
        response = self.client.auth.approle.delete_role(
            role_name=self.TEST_ROLE_NAME, mount_point=self.TEST_MOUNT_POINT
        )

        self.assertEqual(first=204, second=response.status_code)

    def test_read_role_id(self):
        response = self.client.auth.approle.read_role_id(
            role_name=self.TEST_ROLE_NAME, mount_point=self.TEST_MOUNT_POINT
        )

        self.assertEqual(first=self.TEST_ROLE_ID, second=response["data"]["role_id"])

    @parameterized.expand(
        [
            ("good request, no metadata", None, None, None),
            ("good request, good metadata", None, {"a": "val1", "B": "two"}, 300),
            ("good request, good metadata", None, {"a": "val1", "B": "two"}, "5m"),
            ("bad metadata option", exceptions.ParamValidationError, "bad", None),
        ]
    )
    def test_generate_secret_id(self, test_label, raises, metadata, wrap_ttl):
        if raises is not None:
            with self.assertRaises(raises) as cm:
                self.client.auth.approle.generate_secret_id(
                    role_name=self.TEST_ROLE_NAME,
                    metadata=metadata,
                    mount_point=self.TEST_MOUNT_POINT,
                    wrap_ttl=wrap_ttl,
                )
            self.assertIn(
                member="unsupported metadata argument", container=str(cm.exception)
            )
        else:
            response = self.client.auth.approle.generate_secret_id(
                role_name=self.TEST_ROLE_NAME,
                cidr_list=["127.0.0.1/32"],
                mount_point=self.TEST_MOUNT_POINT,
                metadata=metadata,
                wrap_ttl=wrap_ttl,
            )
            if wrap_ttl is not None:
                assert "wrap_info" in response
                assert isinstance(response["wrap_info"]["ttl"], int)
                assert (
                    response["wrap_info"]["ttl"] == 300
                )  # NOTE: hardcoded for now because of string formats
            else:
                self.assertIn(
                    member="secret_id", container=response["data"], msg=response
                )

    @parameterized.expand(
        [
            ("good request, no metadata", None, None, None),
            ("good request, good metadata", None, {"a": "val1", "B": "two"}, 300),
            ("good request, good metadata", None, {"a": "val1", "B": "two"}, "5m"),
            ("bad metadata option", exceptions.ParamValidationError, "bad", None),
        ]
    )
    def test_create_custom_secret_id(self, test_label, raises, metadata, wrap_ttl):
        if raises is not None:
            with self.assertRaises(raises) as cm:
                self.client.auth.approle.create_custom_secret_id(
                    role_name=self.TEST_ROLE_NAME,
                    secret_id=self.TEST_SECRET_ID,
                    cidr_list=["127.0.0.1/32"],
                    metadata=metadata,
                    mount_point=self.TEST_MOUNT_POINT,
                    wrap_ttl=wrap_ttl,
                )
            self.assertIn(
                member="unsupported metadata argument", container=str(cm.exception)
            )
        else:
            response = self.client.auth.approle.create_custom_secret_id(
                role_name=self.TEST_ROLE_NAME,
                secret_id=self.TEST_SECRET_ID,
                cidr_list=["127.0.0.1/32"],
                mount_point=self.TEST_MOUNT_POINT,
                metadata=metadata,
                wrap_ttl=wrap_ttl,
            )
            if wrap_ttl is not None:
                assert "wrap_info" in response
                assert isinstance(response["wrap_info"]["ttl"], int)
                assert (
                    response["wrap_info"]["ttl"] == 300
                )  # NOTE: hardcoded for now because of string formats
            else:
                self.assertEqual(
                    first=self.TEST_SECRET_ID, second=response["data"]["secret_id"]
                )

    def test_read_secret_id(self):
        secret_id_response = self._secret_id()

        response = self.client.auth.approle.read_secret_id(
            role_name=self.TEST_ROLE_NAME,
            secret_id=secret_id_response["secret_id"],
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.assertEqual(first=0, second=response["data"]["secret_id_num_uses"])

    def test_destroy_secret_id(self):
        secret_id_response = self._secret_id()

        response = self.client.auth.approle.destroy_secret_id(
            role_name=self.TEST_ROLE_NAME,
            secret_id=secret_id_response["secret_id"],
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.assertEqual(first=204, second=response.status_code)

    def test_list_secret_id_accessors(self):
        self._secret_id()

        response = self.client.auth.approle.list_secret_id_accessors(
            role_name=self.TEST_ROLE_NAME, mount_point=self.TEST_MOUNT_POINT
        )

        self.assertEqual(first=1, second=len(response["data"]["keys"]))

    def test_read_secret_id_accessor(self):
        secret_id_response = self._secret_id()

        response = self.client.auth.approle.read_secret_id_accessor(
            role_name=self.TEST_ROLE_NAME,
            secret_id_accessor=secret_id_response["secret_id_accessor"],
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.assertEqual(
            first=secret_id_response["secret_id_accessor"],
            second=response["data"]["secret_id_accessor"],
        )
        self.assertEqual(first="127.0.0.1/32", second=response["data"]["cidr_list"][0])

    def test_destroy_secret_id_accessor(self):
        secret_id_response = self._secret_id()

        response = self.client.auth.approle.read_secret_id_accessor(
            role_name=self.TEST_ROLE_NAME,
            secret_id_accessor=secret_id_response["secret_id_accessor"],
            mount_point=self.TEST_MOUNT_POINT,
        )
        self.assertEqual(
            first=secret_id_response["secret_id_accessor"],
            second=response["data"]["secret_id_accessor"],
        )

        response = self.client.auth.approle.destroy_secret_id_accessor(
            role_name=self.TEST_ROLE_NAME,
            secret_id_accessor=secret_id_response["secret_id_accessor"],
            mount_point=self.TEST_MOUNT_POINT,
        )
        self.assertEqual(first=204, second=response.status_code)

    def test_login(self):
        secret_id_response = self._secret_id()

        response = self.client.auth.approle.login(
            role_id=self.TEST_ROLE_ID,
            secret_id=secret_id_response["secret_id"],
            mount_point=self.TEST_MOUNT_POINT,
        )

        self.assertIn(member="client_token", container=response["auth"])
